@@ -5,6 +5,8 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE CPP #-}
+
 module Annex.Content (
 	inAnnex,
 	inAnnexSafe,
@@ -26,11 +28,11 @@ module Annex.Content (
 	preseedTmp,
 	freezeContent,
 	thawContent,
-	replaceFile,
 	cleanObjectLoc,
 ) where
 
 import System.IO.Unsafe (unsafeInterleaveIO)
+import System.PosixCompat.Files
 
 import Common.Annex
 import Logs.Location
@@ -50,6 +52,7 @@ import Git.SharedRepository
 import Annex.Perms
 import Annex.Link
 import Annex.Content.Direct
+import Annex.ReplaceFile
 
 {- Checks if a given key's content is currently present. -}
 inAnnex :: Key -> Annex Bool
@@ -84,14 +87,22 @@ inAnnexSafe = inAnnex' (fromMaybe False) (Just False) go
   where
 	go f = liftIO $ openforlock f >>= check
 	openforlock f = catchMaybeIO $
+#ifndef __WINDOWS__
 		openFd f ReadOnly Nothing defaultFileFlags
+#else
+		return ()
+#endif
 	check Nothing = return is_missing
 	check (Just h) = do
+#ifndef __WINDOWS__
 		v <- getLock h (ReadLock, AbsoluteSeek, 0, 0)
 		closeFd h
 		return $ case v of
 			Just _ -> is_locked
 			Nothing -> is_unlocked
+#else
+		return is_unlocked
+#endif
 	is_locked = Nothing
 	is_unlocked = Just True
 	is_missing = Just False
@@ -100,6 +111,9 @@ inAnnexSafe = inAnnex' (fromMaybe False) (Just False) go
  - it. (If the content is not present, no locking is done.) -}
 lockContent :: Key -> Annex a -> Annex a
 lockContent key a = do
+#ifdef __WINDOWS__
+	a
+#else
 	file <- calcRepo $ gitAnnexLocation key
 	bracketIO (openforlock file >>= lock) unlock a
   where
@@ -121,6 +135,7 @@ lockContent key a = do
 			Right _ -> return $ Just fd
 	unlock Nothing = noop
 	unlock (Just l) = closeFd l
+#endif
 
 {- Runs an action, passing it a temporary filename to get,
  - and if the action succeeds, moves the temp file into 
@@ -230,49 +245,40 @@ moveAnnex :: Key -> FilePath -> Annex ()
 moveAnnex key src = withObjectLoc key storeobject storedirect
   where
 	storeobject dest = ifM (liftIO $ doesFileExist dest)
-		( liftIO $ removeFile src
+		( alreadyhave
 		, do
 			createContentDir dest
 			liftIO $ moveFile src dest
 			freezeContent dest
 			freezeContentDir dest
 		)
-	storedirect fs = storedirect' =<< filterM validsymlink fs
-	validsymlink f = (==) (Just key) <$> isAnnexLink f
+	storeindirect = storeobject =<< calcRepo (gitAnnexLocation key)
 
-	storedirect' [] = storeobject =<< calcRepo (gitAnnexLocation key)
-	storedirect' (dest:fs) = do
+	{- In direct mode, the associated file's content may be locally
+	 - modified. In that case, it's preserved. However, the content
+	 - we're moving into the annex may be the only extant copy, so
+	 - it's important we not lose it. So, when the key's content
+	 - cannot be moved to any associated file, it's stored in indirect
+	 - mode.
+	 -}
+	storedirect = storedirect' storeindirect
+	storedirect' fallback [] = fallback
+	storedirect' fallback (f:fs) = do
 		thawContentDir =<< calcRepo (gitAnnexLocation key)
-		updateInodeCache key src
 		thawContent src
-		replaceFile dest $ liftIO . moveFile src
-		{- Copy to any other locations. -}
-		forM_ fs $ \f -> replaceFile f $
-			liftIO . void . copyFileExternal dest
-
-{- Replaces a possibly already existing file with a new version, 
- - atomically, by running an action.
-
- - The action is passed a temp file, which it can write to, and once
- - done the temp file is moved into place.
- -}
-replaceFile :: FilePath -> (FilePath -> Annex ()) -> Annex ()
-replaceFile file a = do
-	tmpdir <- fromRepo gitAnnexTmpDir
-	createAnnexDirectory tmpdir
-	tmpfile <- liftIO $ do
-		(tmpfile, h) <- openTempFileWithDefaultPermissions tmpdir $
-			takeFileName file
-		hClose h
-		return tmpfile
-	a tmpfile
-	liftIO $ do
-		r <- tryIO $ rename tmpfile file
-		case r of
-			Left _ -> do
-				createDirectoryIfMissing True $ parentDir file
-				rename tmpfile file
-			_ -> noop
+		v <- isAnnexLink f
+		if (Just key == v)
+			then do
+				updateInodeCache key src
+				replaceFile f $ liftIO . moveFile src
+				forM_ fs $
+					addContentWhenNotPresent key f
+			else ifM (goodContent key f)
+				( storedirect' alreadyhave fs
+				, storedirect' fallback fs
+				)
+	
+	alreadyhave = liftIO $ removeFile src
 
 {- Runs an action to transfer an object's content.
  -
@@ -470,9 +476,8 @@ preseedTmp key file = go =<< inAnnex key
 			liftIO $ copyFileExternal s file
 		)
 
-{- Blocks writing to an annexed file. The file is made unwritable
- - to avoid accidental edits. core.sharedRepository may change
- - who can read it. -}
+{- Blocks writing to an annexed file, and modifies file permissions to
+ - allow reading it, per core.sharedRepository setting. -}
 freezeContent :: FilePath -> Annex ()
 freezeContent file = unlessM crippledFileSystem $
 	liftIO . go =<< fromRepo getSharedRepository
@@ -483,7 +488,9 @@ freezeContent file = unlessM crippledFileSystem $
 	go AllShared = modifyFileMode file $
 		removeModes writeModes .
 		addModes readModes
-	go _ = preventWrite file
+	go _ = modifyFileMode file $
+		removeModes writeModes .
+		addModes [ownerReadMode]
 
 {- Allows writing to an annexed file that freezeContent was called on
  - before. -}
