@@ -26,6 +26,7 @@ import Annex.Link
 import Utility.InodeCache
 import Utility.CopyFile
 import Annex.Perms
+import Annex.ReplaceFile
 
 {- Uses git ls-files to find files that need to be committed, and stages
  - them into the index. Returns True if some changes were staged. -}
@@ -157,13 +158,10 @@ mergeDirectCleanup d oldsha newsha = do
 		nukeFile f
 		void $ tryIO $ removeDirectory $ parentDir f
 	
-	{- The symlink is created from the key, rather than moving in the
-	 - symlink created in the temp directory by the merge. This because
-	 - a conflicted merge will write to some other file in the temp
-	 - directory.
-	 -
- 	 - Symlinks are replaced with their content, if it's available. -}
-	movein k f = do
+	{- If the file is already present, with the right content for the
+	 - key, it's left alone. Otherwise, create the symlink and then
+	 - if possible, replace it with the content. -}
+	movein k f = unlessM (goodContent k f) $ do
 		l <- inRepo $ gitAnnexLink f k
 		replaceFile f $ makeAnnexLink l
 		toDirect k f
@@ -191,7 +189,7 @@ toDirectGen k f = do
 		{- Move content from annex to direct file. -}
 		thawContentDir loc
 		updateInodeCache k loc
-		addAssociatedFile k f
+		void $ addAssociatedFile k f
 		thawContent loc
 		replaceFile f $ liftIO . moveFile loc
 	fromdirect = do
@@ -200,18 +198,22 @@ toDirectGen k f = do
 		locs <- filterM (\loc -> isNothing <$> getAnnexLinkTarget loc) =<<
 			(filter (/= absf) <$> addAssociatedFile k f)
 		case locs of
-			(loc:_) -> return $ Just $
+			(loc:_) -> return $ Just $ do
 				replaceFile f $
 					liftIO . void . copyFileExternal loc
+				updateInodeCache k f
 			_ -> return Nothing
 
-{- Removes a direct mode file, while retaining its content. -}
+{- Removes a direct mode file, while retaining its content in the annex
+ - (unless its content has already been changed). -}
 removeDirect :: Key -> FilePath -> Annex ()
 removeDirect k f = do
-	locs <- removeAssociatedFile k f
-	when (null locs) $
-		whenM (isNothing <$> getAnnexLinkTarget f) $
-			moveAnnex k f
+	void $ removeAssociatedFileUnchecked k f
+	unlessM (inAnnex k) $
+		ifM (goodContent k f)
+			( moveAnnex k f
+			, logStatus k InfoMissing
+			)
 	liftIO $ do
 		nukeFile f
 		void $ tryIO $ removeDirectory $ parentDir f
