@@ -38,7 +38,6 @@ import Config
 import qualified Data.Text as T
 import qualified Data.Map as M
 import Data.Char
-import System.Posix.Directory
 
 data RepositoryPath = RepositoryPath Text
 	deriving Show
@@ -138,13 +137,22 @@ getFirstRepositoryR :: Handler RepHtml
 getFirstRepositoryR = postFirstRepositoryR
 postFirstRepositoryR :: Handler RepHtml
 postFirstRepositoryR = page "Getting started" (Just Configuration) $ do
+#ifdef __ANDROID__
+	androidspecial <- liftIO $ doesDirectoryExist "/sdcard/DCIM"
+	let path = "/sdcard/annex"
+#else
+	let androidspecial = False
 	path <- liftIO . defaultRepositoryPath =<< lift inFirstRun
+#endif
 	((res, form), enctype) <- lift $ runFormPost $ newRepositoryForm path
 	case res of
 		FormSuccess (RepositoryPath p) -> lift $
-			startFullAssistant $ T.unpack p
+			startFullAssistant (T.unpack p) ClientGroup
 		_ -> $(widgetFile "configurators/newrepository/first")
 
+getAndroidCameraRepositoryR :: Handler ()
+getAndroidCameraRepositoryR = startFullAssistant "/sdcard/DCIM" SourceGroup
+
 {- Adding a new local repository, which may be entirely separate, or may
  - be connected to the current repository. -}
 getNewRepositoryR :: Handler RepHtml
@@ -291,6 +299,10 @@ driveList = mapM (gen . mnt_dir) =<< filter sane <$> getMounts
 		| dir == "/tmp" = False
 		| dir == "/run/shm" = False
 		| dir == "/run/lock" = False
+#ifdef __ANDROID__
+		| dir == "/mnt/sdcard" = False
+		| dir == "/sdcard" = False
+#endif
 		| otherwise = True
 #else
 driveList = return []
@@ -299,16 +311,15 @@ driveList = return []
 {- Bootstraps from first run mode to a fully running assistant in a
  - repository, by running the postFirstRun callback, which returns the
  - url to the new webapp. -}
-startFullAssistant :: FilePath -> Handler ()
-startFullAssistant path = do
+startFullAssistant :: FilePath -> StandardGroup -> Handler ()
+startFullAssistant path repogroup = do
 	webapp <- getYesod
 	url <- liftIO $ do
 		isnew <- makeRepo path False
 		u <- initRepo isnew True path Nothing
-		inDir path $ 
-			setStandardGroup u ClientGroup
+		inDir path $ setStandardGroup u repogroup
 		addAutoStartFile path
-		changeWorkingDirectory path
+		setCurrentDirectory path
 		fromJust $ postFirstRun webapp
 	redirect $ T.pack url
 
