@@ -6,6 +6,7 @@
  -}
 
 {-# LANGUAGE TypeFamilies, QuasiQuotes, MultiParamTypeClasses, TemplateHaskell, OverloadedStrings, RankNTypes #-}
+{-# LANGUAGE CPP #-}
 
 module Assistant.WebApp.Configurators.Ssh where
 
@@ -64,9 +65,17 @@ sshInputAForm hostnamefield def = SshInput
 	<*> aopt textField "Directory" (Just $ Just $ fromMaybe (T.pack gitAnnexAssistantDefaultDir) $ inputDirectory def)
 	<*> areq intField "Port" (Just $ inputPort def)
   where
+	check_username = checkBool (all (`notElem` "/:@ \t") . T.unpack)
+		bad_username textField
+
+	bad_username = "bad user name" :: Text
+#ifndef __ANDROID__
+	bad_hostname = "cannot resolve host name" :: Text
+
 	check_hostname = checkM (liftIO . checkdns) hostnamefield
 	checkdns t = do
 		let h = T.unpack t
+		let canonname = Just $ defaultHints { addrFlags = [AI_CANONNAME] }
 		r <- catchMaybeIO $ getAddrInfo canonname (Just h) Nothing
 		return $ case catMaybes . map addrCanonName <$> r of
 			-- canonicalize input hostname if it had no dot
@@ -75,13 +84,10 @@ sshInputAForm hostnamefield def = SshInput
 				| otherwise -> Right $ T.pack fullname
 			Just [] -> Right t
 			Nothing -> Left bad_hostname
-	canonname = Just $ defaultHints { addrFlags = [AI_CANONNAME] }
-
-	check_username = checkBool (all (`notElem` "/:@ \t") . T.unpack)
-		bad_username textField
-		
-	bad_hostname = "cannot resolve host name" :: Text
-	bad_username = "bad user name" :: Text
+#else
+	-- getAddrInfo currently broken on Android
+	check_hostname = hostnamefield -- unchecked
+#endif
 
 data ServerStatus
 	= UntestedServer
