@@ -5,6 +5,8 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE CPP #-}
+
 module Remote.Git (
 	remote,
 	configRead,
@@ -18,6 +20,7 @@ import Common.Annex
 import Utility.CopyFile
 import Utility.Rsync
 import Remote.Helper.Ssh
+import Annex.Ssh
 import Types.Remote
 import Types.GitConfig
 import qualified Git
@@ -33,7 +36,7 @@ import qualified Annex.Content
 import qualified Annex.BranchState
 import qualified Annex.Branch
 import qualified Utility.Url as Url
-import Utility.TempFile
+import Utility.Tmp
 import Config
 import Config.Cost
 import Init
@@ -177,7 +180,7 @@ tryGitConfigRead r
 
 	geturlconfig headers = do
 		s <- Url.get (Git.repoLocation r ++ "/config") headers
-		withTempFile "git-annex.tmp" $ \tmpfile h -> do
+		withTmpFile "git-annex.tmp" $ \tmpfile h -> do
 			hPutStr h s
 			hClose h
 			safely $ pipedconfig "git" [Param "config", Param "--null", Param "--list", Param "--file", File tmpfile]
@@ -306,6 +309,8 @@ copyFromRemote' r key file dest
 	 - git-annex-shell transferinfo at the same time
 	 - git-annex-shell sendkey is running.
 	 -
+	 - To avoid extra password prompts, this is only done when ssh
+	 - connection caching is supported.
 	 - Note that it actually waits for rsync to indicate
 	 - progress before starting transferinfo, in order
 	 - to ensure ssh connection caching works and reuses 
@@ -314,7 +319,11 @@ copyFromRemote' r key file dest
 	 - Also note that older git-annex-shell does not support
 	 - transferinfo, so stderr is dropped and failure ignored.
 	 -}
-	feedprogressback a = do
+	feedprogressback a = ifM (isJust <$> sshCacheDir)
+		( feedprogressback' a
+		, a $ const noop
+		)
+	feedprogressback' a = do
 		u <- getUUID
 		let fields = (Fields.remoteUUID, fromUUID u)
 			: maybe [] (\f -> [(Fields.associatedFile, f)]) file
@@ -341,6 +350,7 @@ copyFromRemote' r key file dest
 
 copyFromRemoteCheap :: Remote -> Key -> FilePath -> Annex Bool
 copyFromRemoteCheap r key file
+#ifndef __WINDOWS__
 	| not $ Git.repoIsUrl (repo r) = guardUsable (repo r) False $ do
 		loc <- liftIO $ gitAnnexLocation key (repo r) $
 			fromJust $ remoteGitConfig $ gitconfig r
@@ -350,6 +360,7 @@ copyFromRemoteCheap r key file
 			( copyFromRemote' r key Nothing file
 			, return False
 			)
+#endif
 	| otherwise = return False
 
 {- Tries to copy a key's content to a remote's annex. -}
@@ -396,12 +407,14 @@ rsyncHelper callback params = do
  - filesystem. Then cp could be faster. -}
 rsyncOrCopyFile :: [CommandParam] -> FilePath -> FilePath -> MeterUpdate -> Annex Bool
 rsyncOrCopyFile rsyncparams src dest p =
+#ifdef __WINDOWS__
+	dorsync
+  where
+#else
 	ifM (sameDeviceIds src dest) (docopy, dorsync)
   where
 	sameDeviceIds a b = (==) <$> (getDeviceId a) <*> (getDeviceId b)
 	getDeviceId f = deviceID <$> liftIO (getFileStatus $ parentDir f)
-	dorsync = rsyncHelper (Just p) $
-		rsyncparams ++ [Param src, Param dest]
 	docopy = liftIO $ bracket
 		(forkIO $ watchfilesize zeroBytesProcessed)
 		(void . tryIO . killThread)
@@ -417,6 +430,9 @@ rsyncOrCopyFile rsyncparams src dest p =
 					p sz
 					watchfilesize sz
 			_ -> watchfilesize oldsz
+#endif
+	dorsync = rsyncHelper (Just p) $
+		rsyncparams ++ [File src, File dest]
 
 {- Generates rsync parameters that ssh to the remote and asks it
  - to either receive or send the key's content. -}
