@@ -5,13 +5,15 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE CPP #-}
+
 module Test where
 
 import Test.HUnit
 import Test.QuickCheck
 import Test.QuickCheck.Test
 
-import System.Posix.Files
+import System.PosixCompat.Files
 import Control.Exception.Extensible
 import qualified Data.Map as M
 import System.IO.HVFS (SystemFS(..))
@@ -31,7 +33,6 @@ import qualified Types.KeySource
 import qualified Types.Backend
 import qualified Types.TrustLevel
 import qualified Types
-import qualified GitAnnex
 import qualified Logs.UUIDBased
 import qualified Logs.Trust
 import qualified Logs.Remote
@@ -44,9 +45,9 @@ import qualified Types.Messages
 import qualified Config
 import qualified Config.Cost
 import qualified Crypto
+import qualified Init
 import qualified Utility.Path
 import qualified Utility.FileMode
-import qualified Utility.Gpg
 import qualified Build.SysConfig
 import qualified Utility.Format
 import qualified Utility.Verifiable
@@ -54,6 +55,10 @@ import qualified Utility.Process
 import qualified Utility.Misc
 import qualified Utility.InodeCache
 import qualified Utility.Env
+import qualified Utility.Gpg
+#ifndef __WINDOWS__
+import qualified GitAnnex
+#endif
 
 type TestEnv = M.Map String String
 
@@ -67,15 +72,18 @@ main = do
 	putStrLn "Now, some broader checks ..."
 	putStrLn "  (Do not be alarmed by odd output here; it's normal."
         putStrLn "   wait for the last line to see how it went.)"
-	env <- prepare
-	rs <- forM hunit $ \t -> do
-		divider
-		t env
-	cleanup tmpdir
+	rs <- runhunit =<< prepare False
+	directrs <- runhunit =<< prepare True
 	divider
-	propigate rs qcok
+	propigate (rs++directrs) qcok
   where
 	divider = putStrLn $ replicate 70 '-'
+	runhunit env = do
+		r <- forM hunit $ \t -> do
+			divider
+			t env
+		cleanup tmpdir
+		return r
 
 propigate :: [Counts] -> Bool -> IO ()
 propigate cs qcok
@@ -146,7 +154,8 @@ hunit =
 	, check "status" test_status
 	, check "version" test_version
 	, check "sync" test_sync
-	, check "sync regression" test_sync_regression
+	, check "union merge regression" test_union_merge_regression
+	, check "conflict resolution" test_conflict_resolution
 	, check "map" test_map
 	, check "uninit" test_uninit
 	, check "upgrade" test_upgrade
@@ -165,6 +174,7 @@ hunit =
 test_init :: TestEnv -> Test
 test_init env = "git-annex init" ~: TestCase $ innewrepo env $ do
 	git_annex env "init" [reponame] @? "init failed"
+	handleforcedirect env
   where
 	reponame = "test repo"
 
@@ -206,7 +216,7 @@ test_add env = "git-annex add" ~: TestList [basic, sha1dup, subdirs]
 		git_annex env "add" ["../dir2"] @? "add of ../subdir failed"
 
 test_reinject :: TestEnv -> Test
-test_reinject env = "git-annex reinject/fromkey" ~: TestCase $ intmpclonerepo env $ do
+test_reinject env = "git-annex reinject/fromkey" ~: TestCase $ intmpclonerepoInDirect env $ do
 	git_annex env "drop" ["--force", sha1annexedfile] @? "drop failed"
 	writeFile tmp $ content sha1annexedfile
 	r <- annexeval $ Types.Backend.getKey backendSHA1 $
@@ -221,11 +231,11 @@ test_reinject env = "git-annex reinject/fromkey" ~: TestCase $ intmpclonerepo en
 test_unannex :: TestEnv -> Test
 test_unannex env = "git-annex unannex" ~: TestList [nocopy, withcopy]
   where
-	nocopy = "no content" ~: intmpclonerepo env $ do
+	nocopy = "no content" ~: intmpclonerepoInDirect env $ do
 		annexed_notpresent annexedfile
 		git_annex env "unannex" [annexedfile] @? "unannex failed with no copy"
 		annexed_notpresent annexedfile
-	withcopy = "with content" ~: intmpclonerepo env $ do
+	withcopy = "with content" ~: intmpclonerepoInDirect env $ do
 		git_annex env "get" [annexedfile] @? "get failed"
 		annexed_present annexedfile
 		git_annex env "unannex" [annexedfile, sha1annexedfile] @? "unannex failed"
@@ -330,7 +340,7 @@ test_copy env = "git-annex copy" ~: TestCase $ intmpclonerepo env $ do
 	checkcontent ingitfile
 
 test_lock :: TestEnv -> Test
-test_lock env = "git-annex unlock/lock" ~: intmpclonerepo env $ do
+test_lock env = "git-annex unlock/lock" ~: intmpclonerepoInDirect env $ do
 	-- regression test: unlock of not present file should skip it
 	annexed_notpresent annexedfile
 	not <$> git_annex env "unlock" [annexedfile] @? "unlock failed to fail with not present file"
@@ -358,7 +368,7 @@ test_lock env = "git-annex unlock/lock" ~: intmpclonerepo env $ do
 
 test_edit :: TestEnv -> Test
 test_edit env = "git-annex edit/commit" ~: TestList [t False, t True]
-  where t precommit = TestCase $ intmpclonerepo env $ do
+  where t precommit = TestCase $ intmpclonerepoInDirect env $ do
 	git_annex env "get" [annexedfile] @? "get of file failed"
 	annexed_present annexedfile
 	git_annex env "edit" [annexedfile] @? "edit failed"
@@ -377,7 +387,7 @@ test_edit env = "git-annex edit/commit" ~: TestList [t False, t True]
 	not <$> git_annex env "drop" [annexedfile] @? "drop wrongly succeeded with no known copy of modified file"
 
 test_fix :: TestEnv -> Test
-test_fix env = "git-annex fix" ~: intmpclonerepo env $ do
+test_fix env = "git-annex fix" ~: intmpclonerepoInDirect env $ do
 	annexed_notpresent annexedfile
 	git_annex env "fix" [annexedfile] @? "fix of not present failed"
 	annexed_notpresent annexedfile
@@ -454,14 +464,17 @@ test_fsck env = "git-annex fsck" ~: TestList [basicfsck, barefsck, withlocaluntr
 		git_annex env "get" [f] @? "get of file failed"
 		Utility.FileMode.allowWrite f
 		writeFile f (changedcontent f)
-		not <$> git_annex env "fsck" [] @? "fsck failed to fail with corrupted file content"
+		ifM (annexeval Config.isDirect)
+			( git_annex env "fsck" [] @? "fsck failed in direct mode with changed file content"
+			, not <$> git_annex env "fsck" [] @? "fsck failed to fail with corrupted file content"
+			)
 		git_annex env "fsck" [] @? "fsck unexpectedly failed again; previous one did not fix problem with " ++ f
 	fsck_should_fail m = do
 		not <$> git_annex env "fsck" [] @? "fsck failed to fail with " ++ m
 
 test_migrate :: TestEnv -> Test
 test_migrate env = "git-annex migrate" ~: TestList [t False, t True]
-  where t usegitattributes = TestCase $ intmpclonerepo env $ do
+  where t usegitattributes = TestCase $ intmpclonerepoInDirect env $ do
 	annexed_notpresent annexedfile
 	annexed_notpresent sha1annexedfile
 	git_annex env "migrate" [annexedfile] @? "migrate of not present failed"
@@ -499,21 +512,22 @@ test_migrate env = "git-annex migrate" ~: TestList [t False, t True]
 	checkbackend sha1annexedfile backendSHA256
 
 test_unused :: TestEnv -> Test
-test_unused env = "git-annex unused/dropunused" ~: intmpclonerepo env $ do
+-- This test is broken in direct mode
+test_unused env = "git-annex unused/dropunused" ~: intmpclonerepoInDirect env $ do
 	-- keys have to be looked up before files are removed
 	annexedfilekey <- annexeval $ findkey annexedfile
 	sha1annexedfilekey <- annexeval $ findkey sha1annexedfile
 	git_annex env "get" [annexedfile] @? "get of file failed"
 	git_annex env "get" [sha1annexedfile] @? "get of file failed"
 	checkunused [] "after get"
-	boolSystem "git" [Params "rm -q", File annexedfile] @? "git rm failed"
+	boolSystem "git" [Params "rm -fq", File annexedfile] @? "git rm failed"
 	checkunused [] "after rm"
 	boolSystem "git" [Params "commit -q -m foo"] @? "git commit failed"
 	checkunused [] "after commit"
 	-- unused checks origin/master; once it's gone it is really unused
 	boolSystem "git" [Params "remote rm origin"] @? "git remote rm origin failed"
 	checkunused [annexedfilekey] "after origin branches are gone"
-	boolSystem "git" [Params "rm -q", File sha1annexedfile] @? "git rm failed"
+	boolSystem "git" [Params "rm -fq", File sha1annexedfile] @? "git rm failed"
 	boolSystem "git" [Params "commit -q -m foo"] @? "git commit failed"
 	checkunused [annexedfilekey, sha1annexedfilekey] "after rm sha1annexedfile"
 
@@ -584,10 +598,10 @@ test_sync :: TestEnv -> Test
 test_sync env = "git-annex sync" ~: intmpclonerepo env $ do
 	git_annex env "sync" [] @? "sync failed"
 
-{- Regression test for sync merge bug fixed in
+{- Regression test for union merge bug fixed in
  - 0214e0fb175a608a49b812d81b4632c081f63027 -}
-test_sync_regression :: TestEnv -> Test
-test_sync_regression env = "git-annex sync_regression" ~:
+test_union_merge_regression :: TestEnv -> Test
+test_union_merge_regression env = "union merge regression" ~:
 	{- We need 3 repos to see this bug. -}
 	withtmpclonerepo env False $ \r1 -> do
 		withtmpclonerepo env False $ \r2 -> do
@@ -613,6 +627,47 @@ test_sync_regression env = "git-annex sync_regression" ~:
 					 - thought the file was still in r2 -}
 					git_annex_expectoutput env "find" ["--in", "r2"] []
 
+{- Regression test for the automatic conflict resolution bug fixed
+ - in f4ba19f2b8a76a1676da7bb5850baa40d9c388e2. -}
+test_conflict_resolution :: TestEnv -> Test
+test_conflict_resolution env = "automatic conflict resolution" ~:
+	withtmpclonerepo env False $ \r1 -> do
+		withtmpclonerepo env False $ \r2 -> do
+			let rname r = if r == r1 then "r1" else "r2"
+			forM_ [r1, r2] $ \r -> indir env r $ do
+				{- Get all files, see check below. -}
+				git_annex env "get" [] @? "get failed"
+				{- Set up repos as remotes of each other;
+				 - remove origin since we're going to sync
+				 - some changes to a file. -}
+				when (r /= r1) $
+					boolSystem "git" [Params "remote add r1", File ("../../" ++ r1)] @? "remote add"
+				when (r /= r2) $
+					boolSystem "git" [Params "remote add r2", File ("../../" ++ r2)] @? "remote add"
+				boolSystem "git" [Params "remote rm origin"] @? "remote rm"
+
+				{- Set up a conflict. -}
+				let newcontent = content annexedfile ++ rname r
+				ifM (annexeval Config.isDirect)
+					( writeFile annexedfile newcontent
+					, do
+						git_annex env "unlock" [annexedfile] @? "unlock failed"		
+						writeFile annexedfile newcontent
+					)
+			{- Sync twice in r1 so it gets the conflict resolution
+			 - update from r2 -}
+			forM_ [r1, r2, r1] $ \r -> indir env r $ do
+				git_annex env "sync" [] @? "sync failed in " ++ rname r
+			{- After the sync, it should be possible to get all
+			 - files. This includes both sides of the conflict,
+			 - although the filenames are not easily predictable.
+			 -
+			 - The bug caused, in direct mode, one repo to
+			 - be missing the content of the file that had
+			 - been put in it. -}
+			forM_ [r1, r2] $ \r -> indir env r $ do
+			 	git_annex env "get" [] @? "unable to get all files after merge conflict resolution in " ++ rname r
+
 test_map :: TestEnv -> Test
 test_map env = "git-annex map" ~: intmpclonerepo env $ do
 	-- set descriptions, that will be looked for in the map
@@ -622,7 +677,7 @@ test_map env = "git-annex map" ~: intmpclonerepo env $ do
 	git_annex env "map" ["--fast"] @? "map failed"
 
 test_uninit :: TestEnv -> Test
-test_uninit env = "git-annex uninit" ~: intmpclonerepo env $ do
+test_uninit env = "git-annex uninit" ~: intmpclonerepoInDirect env $ do
 	git_annex env "get" [] @? "get failed"
 	annexed_present annexedfile
 	boolSystem "git" [Params "checkout git-annex"] @? "git checkout git-annex"
@@ -723,7 +778,8 @@ test_bup_remote env = "git-annex bup remote" ~: intmpclonerepo env $ when Build.
 
 -- gpg is not a build dependency, so only test when it's available
 test_crypto :: TestEnv -> Test
-test_crypto env = "git-annex crypto" ~: intmpclonerepo env $ when Build.SysConfig.gpg $ do
+test_crypto env = "git-annex crypto" ~: intmpclonerepo env $ whenM (Utility.Path.inPath Utility.Gpg.gpgcmd) $ do
+#ifndef __WINDOWS__
 	Utility.Gpg.testTestHarness @? "test harness self-test failed"
 	Utility.Gpg.testHarness $ do
 		createDirectory "dir"
@@ -747,12 +803,16 @@ test_crypto env = "git-annex crypto" ~: intmpclonerepo env $ when Build.SysConfi
 		git_annex env "move" [annexedfile, "--from", "foo"] @? "move --from encrypted remote failed"
 		annexed_present annexedfile
 		not <$> git_annex env "drop" [annexedfile, "--numcopies=2"] @? "drop failed to fail"
-		annexed_present annexedfile	
+		annexed_present annexedfile
+#else
+		putStrLn "gpg testing not implemented on Windows"
+#endif
 
 -- This is equivilant to running git-annex, but it's all run in-process
--- so test coverage collection works.
+-- (when the OS allows) so test coverage collection works.
 git_annex :: TestEnv -> String -> [String] -> IO Bool
 git_annex env command params = do
+#ifndef __WINDOWS__
 	forM_ (M.toList env) $ \(var, val) ->
 		Utility.Env.setEnv var val True
 
@@ -763,12 +823,17 @@ git_annex env command params = do
 		Left _ -> return False
   where
 	run = GitAnnex.run (command:"-q":params)
+#else
+	Utility.SafeCommand.boolSystemEnv "git-annex"
+		(map Param $ command : params)
+		(Just $ M.toList env)
+#endif
 
 {- Runs git-annex and returns its output. -}
 git_annex_output :: TestEnv -> String -> [String] -> IO String
 git_annex_output env command params = do
-	got <- Utility.Process.readProcessEnv "git-annex" (command:params) $
-		Just $ M.toList env
+	got <- Utility.Process.readProcessEnv "git-annex" (command:params)
+		(Just $ M.toList env)
 	-- XXX since the above is a separate process, code coverage stats are
 	-- not gathered for things run in it.
 	-- Run same command again, to get code coverage.
@@ -798,6 +863,17 @@ inmainrepo env a = indir env mainrepodir a
 intmpclonerepo :: TestEnv -> Assertion -> Assertion
 intmpclonerepo env a = withtmpclonerepo env False $ \r -> indir env r a
 
+intmpclonerepoInDirect :: TestEnv -> Assertion -> Assertion
+intmpclonerepoInDirect env a = intmpclonerepo env $
+	ifM isdirect
+		( putStrLn "not supported in direct mode; skipping"
+		, a
+		)
+  where
+  	isdirect = annexeval $ do
+		Init.initialize Nothing
+		Config.isDirect
+
 intmpbareclonerepo :: TestEnv -> Assertion -> Assertion
 intmpbareclonerepo env a = withtmpclonerepo env True $ \r -> indir env r a
 
@@ -840,7 +916,14 @@ clonerepo env old new bare = do
 	boolSystem "git" [Params ("clone -q" ++ b), File old, File new] @? "git clone failed"
 	indir env new $
 		git_annex env "init" ["-q", new] @? "git annex init failed"
+	when (not bare) $
+		indir env new $
+			handleforcedirect env
 	return new
+
+handleforcedirect :: TestEnv -> IO ()
+handleforcedirect env = when (M.lookup "FORCEDIRECT" env == Just "1") $
+	git_annex env "direct" ["-q"] @? "git annex direct failed"
 	
 ensuretmpdir :: IO ()
 ensuretmpdir = do
@@ -946,8 +1029,8 @@ annexed_present = runchecks
 unannexed :: FilePath -> Assertion
 unannexed = runchecks [checkregularfile, checkcontent, checkwritable]
 
-prepare :: IO TestEnv
-prepare = do
+prepare :: Bool -> IO TestEnv
+prepare forcedirect = do
 	whenM (doesDirectoryExist tmpdir) $
 		error $ "The temporary directory " ++ tmpdir ++ " already exists; cannot run test suite."
 
@@ -967,6 +1050,7 @@ prepare = do
 		, ("GIT_COMMITTER_NAME", "git-annex test")
 		-- force gpg into batch mode for the tests
 		, ("GPG_BATCH", "1")
+		, ("FORCEDIRECT", if forcedirect then "1" else "")
 		]
 
 	return $ M.fromList env
